#ifndef PATHS_H

#define PATHS_H

/* paths.h
   part of Squirm

   Copyright (C) 1998 Chris Foote & Wayne Piekarski
*/


/* Debugging */
/* #define DEBUG 1 */

/*************  Log File Locations (must be writable by **********/
/*************  the user given in the squid.conf file   **********/
#define LOG_MATCH "/usr/local/squirm/logs/squirm.match"
#define LOG_FAIL  "/usr/local/squirm/logs/squirm.fail"
#define LOG_ERROR "/usr/local/squirm/logs/squirm.error"
#define LOG_WHERE "/usr/local/squirm/logs/squirm.where"
#define LOG_DEBUG "/usr/local/squirm/logs/squirm.debug"
#define LOG_INFO  "/usr/local/squirm/logs/squirm.info"

/*************  Configuration file locations  ***********/
#define LOCAL_ADDRESSES   "/usr/local/squirm/etc/squirm.local"
#define REDIRECT_PATTERNS "/usr/local/squirm/etc/squirm.patterns"

#endif
